 /*
 *********************************************************
 *   Copyright 2003, CyberTAN  Inc.  All Rights Reserved *
 *********************************************************
 
 This is UNPUBLISHED PROPRIETARY SOURCE CODE of CyberTAN Inc.
 the contents of this file may not be disclosed to third parties,
 copied or duplicated in any form without the prior written
 permission of CyberTAN Inc.
 
 This software should be used as a reference only, and it not
 intended for production use!
                                                                                                                             

 THIS SOFTWARE IS OFFERED "AS IS", AND CYBERTAN GRANTS NO WARRANTIES OF ANY
 KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE.  CYBERTAN
 SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE
*/

/*******************************************************************
*
* FILENAME: adm6996i.c
* ABSTRACT: Writing DATA to ADM6996I EEPROM Register 
* AUTHOR  : Zhanghaisen,CyberTAN Technology(GZ),2006-01-10
*
********************************************************************/

#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/slab.h>
#include <linux/ioport.h>
#include <linux/fcntl.h>
#include <linux/interrupt.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/system.h>
#include <linux/delay.h>
#include <linux/devfs_fs_kernel.h>
#include <linux/wait.h>
#include <linux/proc_fs.h>
#include <asm/avalanche/generic/pal.h>
#include <linux/timer.h>

/* 
 *MACRO for ADM6996I EEprom Register
 */
#define MAX_PORT                                        5
#define SIGNATURE                                       0x0
#define PORT0_CONFIG                                    0x1
#define PORT1_CONFIG                            	0x3
#define PORT2_CONFIG                          		0x5
#define PORT3_CONFIG                            	0x7
#define PORT4_CONFIG                           		0x8
#define MII_PORT_CONFIG                        		0x9
#define MII_REGISTER_PORT_MII         			0xb
#define VLAN_PRIORITY_MAP                      		0xe
#define TOS_PRIORITY_MAP                       		0xf
#define MISCELLANEOUS_CONFIG_0      			0x10
#define MAC_CLONE_CONFIG                       		0x11
#define MISCELLANEOUS_CONFIG_1         			0x12
#define VLAN0_OUTBOUND_PORT_MAP        			0x13
#define VLAN1_OUTBOUND_PORT_MAP        			0x14
#define VLAN2_OUTBOUND_PORT_MAP        			0x15
#define VLAN3_OUTBOUND_PORT_MAP        			0x16
#define VLAN4_OUTBOUND_PORT_MAP       			0x17
#define VLAN5_OUTBOUND_PORT_MAP         		0x18
#define VLAN6_OUTBOUND_PORT_MAP        			0x19
#define VLAN7_OUTBOUND_PORT_MAP       			0x1a
#define VLAN8_OUTBOUND_PORT_MAP        			0x1b
#define VLAN9_OUTBOUND_PORT_MAP        			0x1c
#define VLAN10_OUTBOUND_PORT_MAP       			0x1d
#define VLAN11_OUTBOUND_PORT_MAP       			0x1e
#define VLAN12_OUTBOUND_PORT_MAP       			0x1f
#define VLAN13_OUTBOUND_PORT_MAP       			0x20
#define VLAN14_OUTBOUND_PORT_MAP       			0x21
#define VLAN15_OUTBOUND_PORT_MAP        		0x22
#define P0_BUFFER_THRESHOLD                    		0x23
#define P1_BUFFER_THRESHOLD                    		0x24
#define P2_BUFFER_THRESHOLD                    		0x25
#define P3_4_BUFFER_THRESHOLD           		0x26
#define TOTAL_MII_BUFFER_THRESHOLD     			0x27
/*
 *Register Field
 */             
//MAC_CLONE_CONFIG
#define ENABLE_MAC_CLONE         	 0x10
#define MODE_8021Q                       0x20
                                                                         
/*
 *port1-4 as LAN
 */
#define PORT_BASE_CFG            0x800f
#define OUT_TAG_OFFSET           4      
#define PVID_OFFSET              10
#define LAN1_PORT_CONFIG         0x840f  //PVID=1,Untag
#define LAN2_PORT_CONFIG         0x880f  //PVID=2,Untag
#define LAN3_PORT_CONFIG         0x8c0f  //PVID=3,Untag
#define LAN4_PORT_CONFIG         0x900f  //PVID=4,Untag
#define WAN_PORT_CONFIG          0x880f  //PVID=2,Untag
#define CPU_PORT_CONFIG          0x941F  // port 5 PVID = 5, tag
#define VLAN1_PORT_MAP           0x0104  // Port 1+Port5
#define VLAN2_PORT_MAP           0x0110  // Port 2+Port5
#define VLAN3_PORT_MAP           0x0140  // Port 3+Port5
#define VLAN4_PORT_MAP           0x0180  // Port 4+Port5
#define OTHER_VLAN_PORT_MAP      0x0000

#define ZHS_DEBUG
#ifdef ZHS_DEBUG
#define zhs_debug printk
#else
#define zhs_debug(fmt,arg...)
#endif
/*
 *GPIO MACRO
 */	
#define EE_RESET 10// GPIO10
#define EEDO_PIN 17//GPIO17
#define EECS_PIN 9//GPIO9
#define EEDI_PIN 15//GPIO15
#define EECK_PIN 19//GPIO19
                                                                                      
#define EEDO_SHIFT 0
#define EECS_SHIFT 1
#define EEDI_SHIFT 2
#define EECK_SHIFT 3
                                                                                      
#define EEDO_SET (1<< EEDO_SHIFT)  //1 (1B)
#define EECS_SET (1<< EECS_SHIFT)  //2 (10B)
#define EEDI_SET (1<< EEDI_SHIFT)  //4 (100B)
#define EECK_SET (1<< EECK_SHIFT)  //8 (1000B)

#if 0
/* For 7300 */
#define WriteGPIOData(pin,val) avalanche_gpio_out_bit(pin,val)
#else
/* For 7200 */
#define WriteGPIOData(pin,val) PAL_sysGpioOutBit(pin, val)
#endif

#define  conf_gpio(x)     \
           WriteGPIOData(EEDO_PIN, (x&(1<<EEDO_SHIFT))>>EEDO_SHIFT);  \
           WriteGPIOData(EECS_PIN, (x&(1<<EECS_SHIFT))>>EECS_SHIFT);  \
           WriteGPIOData(EEDI_PIN, (x&(1<<EEDI_SHIFT))>>EEDI_SHIFT);  \
           WriteGPIOData(EECK_PIN, (x&(1<<EECK_SHIFT))>>EECK_SHIFT);  \

extern int avalanche_gpio_out_bit(unsigned int pin,int val);
void write_eeprom(short,unsigned short *,int);
void SetEEpromToSendState(void);
void ResetEEpromToSendState(void);
void SetCSToLowForEEprom(void);
void SetCSToHighForEEprom(void);
void send_1ToEEprom(void);
void send_0ToEEprom(void);
//void WriteWait(void);
void SendAddrToEEprom(short);
void SendDataToEEprom(short);
void wr_eeprom(short reg,unsigned short val)
{
	zhs_debug("write reg(0x%x) 0x%x\n",reg,val);
	write_eeprom(reg,&val,1);
}

#if 0
/* For 7300 */
#define  init_vlan_gpio() \
{ \
	 avalanche_gpio_ctrl(EE_RESET,GPIO_PIN,GPIO_OUTPUT_PIN);\
	 avalanche_gpio_ctrl(EEDO_PIN,GPIO_PIN,GPIO_INPUT_PIN);\
	 avalanche_gpio_ctrl(EEDI_PIN,GPIO_PIN,GPIO_OUTPUT_PIN);\
	 avalanche_gpio_ctrl(EECS_PIN,GPIO_PIN,GPIO_OUTPUT_PIN);\
	 avalanche_gpio_ctrl(EECK_PIN,GPIO_PIN,GPIO_OUTPUT_PIN);\
}
#else
#define  init_vlan_gpio() \
{ \
	 PAL_sysGpioCtrl(EE_RESET,GPIO_PIN,GPIO_OUTPUT_PIN);\
	 PAL_sysGpioCtrl(EEDO_PIN,GPIO_PIN,GPIO_INPUT_PIN);\
	 PAL_sysGpioCtrl(EEDI_PIN,GPIO_PIN,GPIO_OUTPUT_PIN);\
	 PAL_sysGpioCtrl(EECS_PIN,GPIO_PIN,GPIO_OUTPUT_PIN);\
	 PAL_sysGpioCtrl(EECK_PIN,GPIO_PIN,GPIO_OUTPUT_PIN);\
}

#endif
void reset_adm6996I(void)
{
//	WriteGPIOData(EE_RESET,1);
	int i;
	printk("reset_adm6996I!\n");
	WriteGPIOData(EE_RESET,1);
	for (i = 0 ; i < 5000000 ; i ++);
	WriteGPIOData(EE_RESET,0);
	for (i = 0 ; i < 10000000 ; i ++);
	WriteGPIOData(EE_RESET,1);
	for (i = 0 ; i < 10000000 ; i ++);

	
}

/* all function below is used for ADM6996I register */
#define getPortCfgByVID(vid,isTag) PORT_BASE_CFG | ((vid)<<PVID_OFFSET)  | (isTag<<OUT_TAG_OFFSET)

#define getPortCfgAddr0_3(n)           PORT0_CONFIG + n*2

#define SetPortCfg(n,vid,isTag) \
{\
/*	value = LAN##n##_PORT_CONFIG;*/ \
	unsigned short value = getPortCfgByVID(vid,isTag);\
	short port_addr = getPortCfgAddr(n);\
	zhs_debug("set port%d(addr:0x%x) 0x%x\n",n,port_addr,value);\
	write_eeprom(port_addr,&value,0x1);\
}

#define getVLANMap(p0,p1,p2,p3,p4,p5) (p0)|(p1<<2)|(p2<<4)|(p3<<6)|(p4<<7)|(p5<<8)

#define SetVLANPortMap(n,p0,p1,p2,p3,p4,p5) \
{\
	unsigned short value = getVLANMap(p0,p1,p2,p3,p4,p5);\
	zhs_debug("set VLANGroup%d(addr:0x%x) 0x%x\n",n,VLAN0_OUTBOUND_PORT_MAP+n,value);\
	write_eeprom(VLAN0_OUTBOUND_PORT_MAP+n,&value,0x1);\
}

short getPortCfgAddr(int n)
{
        if (n<4)
                return getPortCfgAddr0_3(n);
        if(n==4)
                return 8;
        if(n==5)
                return 9;
        return 0;
}


void set_auto_mdix(void)
{
	int i;
	unsigned short value;	

	/*set Port config*/	
	for(i=1;i<=MAX_PORT;i++)
	{
		SetPortCfg(i,0,0);	
	}

}

/**************************************************************************
*
* NAME		: write_eeprom
* DESCRIPTION	: Write specified data to eeprom
* INPUT		: RegNumber	--- Register Number will be written
* 		  data		--- data will be written into register
* 		  len		--- number of short(2 bytes) to be written
* OUTPUT	: None
*
**************************************************************************/
void write_eeprom(short RegNumber,unsigned short *data, int len )
{
    int  i2;
    short s_addr, s_data ;
    short *src;

    src = data;
    SetEEpromToSendState();
//  the write enable(WEN) instruction must be executed before any device
//  programming can be done

    s_data = 0x04c0;
    SendAddrToEEprom(s_data); //00000001 0011000000B
    SetCSToLowForEEprom();

    s_addr = 0x0500 | (RegNumber & 0x0ff); //00000001 01dddddddd
    s_data = *src;

    for (i2 = len; i2 > 0 ; i2 --)
    {
      SendAddrToEEprom(s_addr); //00000001 01dddddd
      SendDataToEEprom(s_data); //dddddddd dddddddd
      SetCSToLowForEEprom();
      SetCSToLowForEEprom();
      //WriteWait();     
      s_addr ++;
      src ++;
      s_data = *src;
    }
//  after all data has been written to EEprom , the write disable(WDS) 
//  instruction must be executed
    SetCSToHighForEEprom();
    s_data = 0x0400;
    SendAddrToEEprom(s_data); //00000001 00000000B
    SetCSToLowForEEprom();
    SetCSToLowForEEprom();
}

void SetEEpromToSendState()
{
  conf_gpio(0x0);
  conf_gpio(0x0);
  conf_gpio(EECS_SET);
  conf_gpio(EECS_SET);
 
//  conf_gpio(0x0);
//  conf_gpio(0x0);
//  conf_gpio(0x40);
//  conf_gpio(0x40);
  
//  ;cs     __--   ,bit 6
//  ;sck    ____   ,bit 7
//  ;di     ____   ,bit 18
//  ;do     ____   ,bit 19
//  ;
}

void ResetEEpromToSendState()
{
  conf_gpio(0x0);
  conf_gpio(0x0);
  conf_gpio(0x0);
  conf_gpio(0x0);
  
//  ;cs     ____   ,bit 6  
//  ;sck    ____   ,bit 7
//  ;di     ____   ,bit 18
//  ;do     ____   ,bit 19 
//  ;
}

void SetCSToLowForEEprom()
{	
  conf_gpio(0x0);
  conf_gpio(EECK_SET);
  conf_gpio(EECK_SET);
  conf_gpio(0x0);

//  conf_gpio(0x0);
//  conf_gpio(0x80);
//  conf_gpio(0x80);
//  conf_gpio(0x0);
  
//  ;cs     ____   ,bit 6
//  ;sck    _--_   ,bit 7
//  ;di     ____   ,bit 18
//  ;do     ____   ,bit 19 
//  ;
}

void SetCSToHighForEEprom()
{
  conf_gpio(EECS_SET);
  conf_gpio(EECS_SET + EECK_SET);
  conf_gpio(EECS_SET + EECK_SET);
  conf_gpio(EECS_SET);

//  conf_gpio(0x40);
//  conf_gpio(0xc0);
//  conf_gpio(0xc0);
//  conf_gpio(0x40);
  
//  ;cs     ----   ,bit 6
//  ;sck    _--_   ,bit 7
//  ;di     ____   ,bit 18
//  ;do     ____   ,bit 19  
//  ;
}

void send_1ToEEprom()
{
  conf_gpio(EECS_SET + EEDI_SET);
  conf_gpio(EECS_SET + EECK_SET + EEDI_SET);
  conf_gpio(EECS_SET + EECK_SET + EEDI_SET);
  conf_gpio(EECS_SET + EEDI_SET);

//  conf_gpio(0x40040);
//  conf_gpio(0x400c0);
//  conf_gpio(0x400c0);
//  conf_gpio(0x40040);
  
//  ;cs     ----   ,bit 6
//  ;sck    _--_   ,bit 7
//  ;di     ----   ,bit 18
//  ;do     ____   ,bit 19
//  ;       
}

void send_0ToEEprom()
{
  conf_gpio(EECS_SET);
  conf_gpio(EECS_SET + EECK_SET);
  conf_gpio(EECS_SET + EECK_SET);
  conf_gpio(EECS_SET);	
	
//  conf_gpio(0x40);
//  conf_gpio(0xc0);
//  conf_gpio(0xc0);
//  conf_gpio(0x40);
  
//  ;cs     ----   ,bit 6
//  ;sck    _--_   ,bit 7
//  ;di     ____   ,bit 18
//  ;do     ____   ,bit 19
//  ;       
}

#if 0
void WriteWait()
{
  unsigned int status;
  
  SetCSToLowForEEprom();
  SetCSToHighForEEprom();
  do {
  	SetCSToHighForEEprom();
  	status = ReadGPIOData(EEDO_PIN);
  }
  while (!status);   // wait for write - ready
  SetCSToLowForEEprom();
}
#endif

void SendDataToEEprom(short s_data) 
{
  int data_mask;
  
  for (data_mask = 0x8000; data_mask != 0; )
  {
    if (s_data & data_mask) 
      send_1ToEEprom();
    else
      send_0ToEEprom();  
    data_mask = data_mask >> 1;
  }
}

void SendAddrToEEprom(short s_data) 
{
  int data_mask;
  
  for (data_mask = 0x0400 ;data_mask != 0; )
  {
    if (s_data & data_mask) 
      send_1ToEEprom();
    else
      send_0ToEEprom();  
    data_mask = data_mask >> 1;
  }
}
/* Define the major number and device name for ADM6996I */ 
#define SWI_MAJOR_DEVICE_NUMBER 69        
#define SWI_DEVICE_MODULE_NAME  "adm6996" 

#define ADM_CMD_WR                   1
#define ADM_CMD_RST                  2
#define REG_ADDR_MAX                 0xff

int adm_open(struct inode *node, struct file *filp)
{
                                                                                
    MOD_INC_USE_COUNT;
                                                                                
    return 0;
}
                                                                                
ssize_t adm_read(struct file *filep, char *buf, size_t count, loff_t *ppos)
{
                                                                                
    return count;
}
                                                                                
ssize_t adm_write(struct file *filep, const char *buf, size_t count, loff_t *ppos)
{

    return count;
}
                                                                                
/* close */
int adm_release(struct inode *inode, struct file *filp)
{
                                                                                
    MOD_DEC_USE_COUNT;
                                                                                
    return 0;
}
typedef struct reg_write{
	unsigned short reg_val;
	unsigned short reg_addr;
	}reg_wr;
                                                                                
/* IOCTL function */
int adm_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long args)
{
	reg_wr reg;
	switch(cmd)
	{
		case ADM_CMD_WR:
			if(copy_from_user(&reg,(reg_wr *)args,sizeof(reg_wr)))
			{
				printk("adm_ioctl():copy from user failed!!\n");
				return (-EFAULT);
			}
			if(reg.reg_addr > REG_ADDR_MAX)
			{
				printk("adm_ioctl():reg address(%d) overflow!!\n",reg.reg_addr);
				return (-EFAULT);
			}
			wr_eeprom(reg.reg_addr,reg.reg_val);
			break;		
		case ADM_CMD_RST:
			reset_adm6996I();
			break;		
		default:
			printk("adm_ioctl():unkown command(%d)!!\n",cmd);
			return (-EFAULT);
	}
		return 0;				
}
struct file_operations adm_ops = {
        read: adm_read,
        write: adm_write,
        open: adm_open,
        release: adm_release,
        ioctl: adm_ioctl
};
static devfs_handle_t adm_devfs_handle =NULL;
int __init init_adm_sw(void)
{
	 printk("Loading ADM6996I device driver...\n");
                                                                                
	init_vlan_gpio()                                                                                
        reset_adm6996I();
	set_auto_mdix();
 	 
	adm_devfs_handle = devfs_register(NULL, SWI_DEVICE_MODULE_NAME, DEVFS_FL_AUTO_DEVNUM, 0, 0,
                        S_IFCHR | S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP, 
 		        &adm_ops,NULL);

                                                                               
                                                                                
  return 0;
}

void  cleanup_adm_sw(void)
{
	printk("Unloading ADM6996I device driver...\n");
        devfs_unregister(adm_devfs_handle);                                                                        
}

module_init(init_adm_sw);
module_exit(cleanup_adm_sw);
