/*
 *  System MIB group implementation - system.c
 *
 */
#include <stdlib.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <paths.h>
#include <signal.h>
#include <stdarg.h>



#include <sys/types.h>
#include <sys/un.h>
#include <sys/param.h>

/***********songtao 4.6.2004*******************/


/*********************************************/
/* SYSLOG_NAMES defined to pull some extra junk from syslog.h */
#define SYSLOG_NAMES
#include <sys/syslog.h>
#include <sys/uio.h>

#include <config.h>

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#if HAVE_STRING_H
#include <string.h>
#else
#include <strings.h>
#endif
#include <sys/types.h>
#if HAVE_WINSOCK_H
#include <winsock.h>
#endif

#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif

#include <ctype.h>
#if HAVE_UTSNAME_H
#include <utsname.h>
#else
#if HAVE_SYS_UTSNAME_H
#include <sys/utsname.h>
#endif
#endif
#if HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif

#if HAVE_DMALLOC_H
#include <dmalloc.h>
#endif

#include "mibincl.h"
#include "util_funcs.h"
#include "system_mib.h"
#include "struct.h"
#include "read_config.h"
#include "agent_read_config.h"
#include "system.h"
#include "sysORTable.h"


	/*********************
	 *
	 *  Kernel & interface information,
	 *   and internal forward declarations
	 *
	 *********************/
int nvram_fd1=-1;
#define SYS_STRING_LEN	256

char version_descr[ SYS_STRING_LEN ] = VERS_DESC;
char sysContact[    SYS_STRING_LEN ] = SYS_CONTACT;
char sysName[       SYS_STRING_LEN ] = SYS_NAME;
char sysLocation[   SYS_STRING_LEN ] = SYS_LOC;
/*
char version_descr[ SYS_STRING_LEN ];
char sysContact[    SYS_STRING_LEN ];
char sysName[       SYS_STRING_LEN ];
char sysLocation[   SYS_STRING_LEN ];
*/
char oldversion_descr[ SYS_STRING_LEN ];
char oldsysContact[    SYS_STRING_LEN ];
char oldsysName[       SYS_STRING_LEN ];
char oldsysLocation[   SYS_STRING_LEN ];

int sysServices=72;
int sysServicesConfiged=0;

extern oid version_id[];
extern int version_id_len;


WriteMethod writeSystem;
int header_system(struct variable *,oid *, size_t *, int, size_t *, WriteMethod **);

	/*********************
	 *
	 *  snmpd.conf config parsing
	 *
	 *********************/

void system_parse_config_sysloc(const char *token, 
				char *cptr)
{
  char tmpbuf[1024];
  
  if (strlen(cptr) < sizeof(sysLocation)) {
    strcpy(sysLocation,cptr);
  } else {
    sprintf(tmpbuf, "syslocation token too long (must be < %d):\n\t%s",
		 sizeof(sysLocation), cptr);
    config_perror(tmpbuf);
  }
}

void system_parse_config_sysServices(const char *token, char *cptr)
{
  sysServices = atoi(cptr);
  sysServicesConfiged = 1;
}

void system_parse_config_syscon(const char *token, 
				char *cptr)
{
  char tmpbuf[1024];

  if (strlen(cptr) < sizeof(sysContact)) {
    strcpy(sysContact,cptr);
  } else {
    sprintf(tmpbuf, "syscontact token too long (must be < %d):\n\t%s",
                 sizeof(sysContact), cptr);
    config_perror(tmpbuf);
  }
}


	/*********************
	 *
	 *  Initialisation & common implementation functions
	 *
	 *********************/

/* define the structure we're going to ask the agent to register our
   information at */
struct variable2 system_variables[] = {
    {VERSION_DESCR, ASN_OCTET_STR, RONLY, var_system, 1, {1}},
    {VERSIONID, ASN_OBJECT_ID, RONLY, var_system, 1, {2}},
    {UPTIME, ASN_TIMETICKS, RONLY, var_system, 1, {3}},
    {SYSCONTACT, ASN_OCTET_STR, RWRITE, var_system, 1, {4}},
    {SYSTEMNAME, ASN_OCTET_STR, RWRITE, var_system, 1, {5}},
    {SYSLOCATION, ASN_OCTET_STR, RWRITE, var_system, 1, {6}},
    {SYSSERVICES, ASN_INTEGER, RONLY, var_system, 1, {7}},
    {SYSORLASTCHANGE, ASN_TIMETICKS, RONLY, var_system, 1, {8}}
};
/* Define the OID pointer to the top of the mib tree that we're
   registering underneath */
oid system_variables_oid[] = { SNMP_OID_MIB2,1 };
oid system_module_oid[]    = { SNMP_OID_SNMPMODULES,1 };
int system_module_oid_len  = sizeof( system_module_oid ) / sizeof( oid );
int system_module_count    = 0;

void init_system_mib(void)
{
  //xgz
  char getDevname[SYS_STRING_LEN];
  char getContact[SYS_STRING_LEN];
  char getLocation[SYS_STRING_LEN]; 

#ifdef HAVE_UNAME
  struct utsname utsName;

  uname(&utsName);
  sprintf(version_descr, "%s", "Wireless G ADSL Gateway");
#else
#if HAVE_EXECV
  struct extensible extmp;

  /* set default values of system stuff */
  sprintf(extmp.command,"%s -a",UNAMEPROG);
  /* setup defaults */
  extmp.type = EXECPROC;
  extmp.next = NULL;
  exec_command(&extmp);
  strcpy(version_descr,"Wireless G ADSL Gateway");
  version_descr[strlen(version_descr)-1] = 0; /* chomp new line */
#else
  strcpy(version_descr, "Wireless G ADSL Gateway" );
#endif
#endif
  strcpy(getDevname, nvram_get("devname"));

  //xgz 2005-11-01
  strcpy(getContact, nvram_get("sysContact"));
  strcpy(getLocation, nvram_get("sysLocation"));

  //fprintf(stderr,"getLocation=%s  getContact=%s  dev_str_len=%d\n", getLocation, getContact, strlen(getDevname));
  if( strlen(getDevname) >= SYS_STRING_LEN )
	getDevname[SYS_STRING_LEN-1] = 0;
  else
	getDevname[ strlen(getDevname) ] = 0;

  if( strlen(getContact) >= SYS_STRING_LEN )
	getContact[SYS_STRING_LEN-1] = 0;
  else
	getContact[ strlen(getContact) ] = 0;

  if( strlen(getDevname) >= SYS_STRING_LEN )
	getLocation[SYS_STRING_LEN-1] = 0;
  else
	getLocation[ strlen(getLocation) ] = 0;
  //fprintf(stderr,"getLocation2=%s  getContact2=%s  dev_str_len2=%d\n", getLocation, getContact, strlen(getDevname));


#if 1
  if( strcmp(getContact, "") )
	strcpy(sysContact, getContact);
  else
	strcpy(sysContact, SYS_CONTACT);

  if( strcmp(getLocation, "") )
	strcpy(sysLocation, getLocation);
  else
	strcpy(sysLocation, SYS_LOC);

  //fprintf(stderr,"sysLocation=%s  sysContact=%s\n", sysLocation, sysContact);
  //xgz end

#endif

#ifdef HAVE_GETHOSTNAME
 // gethostname(sysName,sizeof(sysName));
  strcpy(sysName,getDevname);
#else
#ifdef HAVE_UNAME
  strcpy(sysName,getDevname);
#else
#if HAVE_EXECV
  sprintf(extmp.command,"%s -n",UNAMEPROG);
  /* setup defaults */
  extmp.type = EXECPROC;
  extmp.next = NULL;
  exec_command(&extmp);
  strcpy(sysName,getDevname);
  //sysName[strlen(sysName)-1] = 0; /* chomp new line */
#else
  strcpy(sysName, getDevname );
#endif /* HAVE_EXECV */
#endif /* HAVE_UNAME */
#endif /* HAVE_GETHOSTNAME */

  /* register ourselves with the agent to handle our mib tree */
  REGISTER_MIB("mibII/system", system_variables, variable2,
               system_variables_oid);

  if ( ++system_module_count == 3 )
	REGISTER_SYSOR_ENTRY( system_module_oid,
		"The MIB module for SNMPv2 entities");
  
  /* register our config handlers */
  snmpd_register_config_handler("syslocation", system_parse_config_sysloc,
                                NULL, "location");
  snmpd_register_config_handler("syscontact", system_parse_config_syscon,
                                NULL,"contact-name");
  snmpd_register_config_handler("sysservices", system_parse_config_sysServices,
                                NULL,"NUMBER");

}
/*****************songtao 4.6.2004****************************************/
int
nvram_init(void *unused)
{
         if ((nvram_fd1 = open(PATH_DEV_NVRAM, O_RDWR)) < 0)
                goto err;
         return 0;
 err:
         perror(PATH_DEV_NVRAM);
         return errno;
}

 char *
 nvram_get(const char *name)
{
       size_t count = strlen(name) + 1;
       char *off;
       memset((void *)nvram_value, 0, MAX_LEN);
       off = (char *)nvram_value;
       if (nvram_fd1 < 0)
          if (nvram_init(NULL))
             return NULL;
       strcpy((char *) off, name);
       count = read(nvram_fd1, off, count);
       return off;
}

//xgz add
#if 1
static int
nvram_set_sys(const char *name, const char *value)
{
        size_t count = strlen(name) + 1;
        char tmp[100], *buf = tmp;
        int ret;
                                                                                
        if (nvram_fd1 < 0)
                   if ((ret = nvram_init(NULL)))
                                  return ret;
       /* Unset if value is NULL */
        if (value)
                   count += strlen(value) + 1;
        if (count > sizeof(tmp)) {
                   if (!(buf = malloc(count)))
                                  return -ENOMEM;
                 }
        if (value)
                   sprintf(buf, "%s=%s", name, value);
        else
           strcpy(buf, name);
        ret = write(nvram_fd1, buf, count);
        if (ret < 0)
                   perror(PATH_DEV_NVRAM);
        if (buf != tmp)
                   free(buf);
                                                                                
        return (ret == count) ? 0 : ret;
}
#endif

							   
/******************songtao end**********************************************/

	/*********************
	 *
	 *  System specific implementation functions
	 *
	 *********************/

#ifdef USING_MIBII_SYSORTABLE_MODULE
extern struct timeval sysOR_lastchange;
#endif

u_char	*
var_system(struct variable *vp,
	   oid *name,
	   size_t *length,
	   int exact,
	   size_t *var_len,
	   WriteMethod **write_method)
{

    struct timeval now;

    if (header_generic(vp, name, length, exact, var_len, write_method) == MATCH_FAILED )
	return NULL;

    switch (vp->magic){
        case VERSION_DESCR:
            *var_len = strlen(version_descr);
            *write_method = writeSystem;
            return (u_char *)version_descr;
        case VERSIONID:
            *var_len = version_id_len*sizeof(version_id[0]);
            return (u_char *)version_id;
        case UPTIME:
            gettimeofday(&now, NULL);
	    long_return = timeval_uptime( &now );
            return ((u_char *) &long_return);
        case SYSCONTACT:
            *var_len = strlen(sysContact);
            *write_method = writeSystem;
            return (u_char *)sysContact;
        case SYSTEMNAME:
            *var_len = strlen(sysName);
            *write_method = writeSystem;
            return (u_char *)sysName;
        case SYSLOCATION:
            *var_len = strlen(sysLocation);
            *write_method = writeSystem;
            return (u_char *)sysLocation;
        case SYSSERVICES:
#if NO_DUMMY_VALUES
            if (!sysServicesConfiged)
                return NULL;
#endif
            long_return = sysServices;
            return (u_char *)&long_return;

#ifdef USING_MIBII_SYSORTABLE_MODULE
        case SYSORLASTCHANGE:
	      long_return = timeval_uptime( &sysOR_lastchange );
              return ((u_char *) &long_return);
#endif
              
	default:
	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_system\n", vp->magic));
    }
    return NULL;
}



int
writeSystem(int action,	     
	    u_char *var_val,
	    u_char var_val_type,
	    size_t var_val_len,
	    u_char *statP,
	    oid *name,
	    size_t name_len)
{
    u_char *cp;
    char *buf = NULL, *oldbuf = NULL;
    int count;
    //xgz
    char nv_name[50];
    memset(nv_name, 0, sizeof(nv_name));

    switch((char)name[7]){
      case VERSION_DESCR:
        buf    = version_descr;
        oldbuf = oldversion_descr;
        break;
      case SYSCONTACT:
        buf    = sysContact;
        oldbuf = oldsysContact;
	strcpy(nv_name, "sysContact");//xgz
        break;
      case SYSTEMNAME:
        buf    = sysName;
        oldbuf = oldsysName;
	strcpy(nv_name, "devname");//xgz
        break;
      case SYSLOCATION:
        buf    = sysLocation;
        oldbuf = oldsysLocation;
	strcpy(nv_name, "sysLocation");//xgz
        break;
      default:
	return SNMP_ERR_GENERR;		/* ??? */
    }

    switch ( action ) {
	case RESERVE1:		/* Check values for acceptability */
	    if (var_val_type != ASN_OCTET_STR){
                snmp_log(LOG_ERR, "not string\n");
		return SNMP_ERR_WRONGTYPE;
	    }
	    if (var_val_len > sizeof(version_descr)-1){
                snmp_log(LOG_ERR, "bad length\n");
		return SNMP_ERR_WRONGLENGTH;
	    }
	    
	    for(cp = var_val, count = 0; count < (int)var_val_len; count++, cp++){
		if (!isprint(*cp)){
                    snmp_log(LOG_ERR, "not print %x\n", *cp);
		    return SNMP_ERR_WRONGVALUE;
		}
	    }
	    break;

	case RESERVE2:		/* Allocate memory and similar resources */

		/* Using static strings, so nothing needs to be done */
	    break;

	case ACTION:		/* Perform the SET action (if reversible) */

	    //fprintf(stderr, "buf=%s   oldbuf=%s  var_val=%s   var_len=%d\n", buf, oldbuf, var_val, var_val_len);
		/* Save the old value, in case of UNDO */
	    memset(oldbuf, 0, SYS_STRING_LEN);
	    strcpy( oldbuf, buf);
	    //memcpy( buf, var_val, var_val_len);
	    memset(buf, 0, SYS_STRING_LEN);
	    strncpy(buf, var_val, var_val_len);
	    if(var_val_len >= SYS_STRING_LEN)
		buf[SYS_STRING_LEN -1] = 0;
	    else
		buf[ var_val_len ] = 0;
	    //xgz
	    //fprintf(stderr, "buf=%s   oldbuf=%s\n", buf, oldbuf);
	    if( strcmp(oldbuf, buf) )
	    {
	            nvram_set_sys(nv_name, buf);
    		    system("nvram commit");
	    }
            
	    buf[var_val_len] = 0;
	    break;

	case UNDO:		/* Reverse the SET action and free resources */

	    strcpy( buf, oldbuf);
	    oldbuf[0] = 0;
	    //xgz
            //nvram_set_sys(nv_name, buf);
  	    //system("nvram commit");
	    break;

	case COMMIT:		/* Confirm the SET, performing any irreversible actions,
					and free resources */
	case FREE:		/* Free any resources allocated */

		/* No resources have been allocated, but "empty" the 'oldbuf' */
	    oldbuf[0] = 0;
	    break;
    }
    return SNMP_ERR_NOERROR;
} /* end of writeSystem */

	/*********************
	 *
	 *  Internal implementation functions - None
	 *
	 *********************/

