/*
 *********************************************************
 *   Copyright 2003, CyberTAN  Inc.  All Rights Reserved *
 *********************************************************
 
 This is UNPUBLISHED PROPRIETARY SOURCE CODE of CyberTAN Inc.
 the contents of this file may not be disclosed to third parties,
 copied or duplicated in any form without the prior written
 permission of CyberTAN Inc.
 
 This software should be used as a reference only, and it not
 intended for production use!

 THIS SOFTWARE IS OFFERED "AS IS", AND CYBERTAN GRANTS NO WARRANTIES OF ANY
 KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE.  CYBERTAN
 SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <bcmnvram.h>
#include <shutils.h>
#include <utils.h>
#include <rc.h>
#include "rc_igmp_snp.h"

extern int g_reboot;

#define ZB_DEBUG	0

#if ZB_DEBUG
#define D_PRINTF	printf
#else
#define D_PRINTF(x)
#endif

/******************************************************************************
*
*NAME              : start_igmp_snp()
*DESCRIPTION       : start IGMP SNOOPING 
*
*INPUTS            : NONE       
*OUTPUTS(O)        : NONE
*RETURN            : int
*
*modification history
*------------------------------------------------------------------------------
* 1.00, Dec 30, 2005, zhangbin written.
*------------------------------------------------------------------------------
*
*AUTHOR            : zhangbin, CyberTAN Technology(GZ), 2005.12.30
**************************************************************************** */

int start_igmp_snp(void)
{
	char enable[3];

	memset(enable, 0, sizeof(enable));
	sprintf(enable, "%s", nvram_safe_get("igmp_snp_enable"));

#if 0
	if(!strcmp(enable, "0"))
	{
		D_PRINTF("IGMP Snooping is disabled now !!!\n");
		return -1;
	}
	else
#endif
	{
		int fd,i;
	
		fd = open(SWI_DEVICE_MODULE_NAME,O_WRONLY);
		if(-1 == fd)
		{
			printf("%s open %s failed!\n",__FUNCTION__,SWI_DEVICE_MODULE_NAME);
			return -1;
		}
		else
		{
			unsigned short reg_value;
			unsigned int router_map[SWI_MAX_PORT_NUMBER],router_mapval = 0;
			int igmp_enable,ignore_cpuport,qry_interval,robust_var;
			short reg_addr;

			if(!strcmp(enable, "0"))
			{

				for(i = 1;i < SWI_MAX_PORT_NUMBER;i++)	
				{	
					router_map[i] = 0;
				}
					
				igmp_enable = 0;
				ignore_cpuport = 0;
				qry_interval = 0; /* must be in 0-255 */
				robust_var = 0; /* must be in 1-3 */
			}
			else
			{
				/* FIXME:need to set all router_map? 
				* PHY port 0 is not used
				*/
				for(i = 1;i < SWI_MAX_PORT_NUMBER;i++)	
				{	
					router_map[i] = 1;
				}
				
				igmp_enable = 1;
				ignore_cpuport = 1;
				qry_interval = 20; /* must be in 0-255 */
				robust_var = 2; /* must be in 1-3 */
			}

			for(i = 0;i < SWI_MAX_PORT_NUMBER;i++)	
			{	
				if(router_map[i])
				{
					router_mapval |= (1 << i);
				}
			}

			reg_addr = SWI_HWIGMP_CTRLREG;	
			reg_value = qry_interval << SWI_IGMP_QRYINTERVAL_OFFSET;
			reg_value |= (robust_var << SWI_IGMP_ROBUSTVAR_OFFSET);
			reg_value |= router_mapval; 
		
			if(write_adm_file(fd,reg_addr,reg_value))
			{
				printf("start IGMP Snooping failed!\n");
				close(fd);
				return -1;
			}

			if(router_mapval)
				reg_value = SWI_IGMP_ROUTER_ENABLE_BIT;
			else
				reg_value = 0;

			if(igmp_enable)
				reg_value |= SWI_IGMP_SNOOP_ENABLE_BIT;

			if(ignore_cpuport)
				reg_value |= SWI_IGMP_IGNORE_CPUPORT_BIT;

			reg_addr = SWI_MCSTSNOOP_CTRLREG;

			if(write_adm_file(fd,reg_addr,reg_value))
			{
				printf("start IGMP Snooping failed!\n");
				return -1;
			}
				
			close(fd);
				
			D_PRINTF("IGMP Snooping started!\n");

		}
	}

	return 0;
}

/******************************************************************************
*
*NAME              : stop_igmp_snp()
*DESCRIPTION       : stop IGMP SNOOPING 
*
*INPUTS            : NONE       
*OUTPUTS(O)        : NONE
*RETURN            : int
*
*modification history
*------------------------------------------------------------------------------
* 1.00, Dec 30, 2005, zhangbin written.
*------------------------------------------------------------------------------
*
*AUTHOR            : zhangbin, CyberTAN Technology(GZ), 2005.12.30
**************************************************************************** */

int stop_igmp_snp(void)
{
#if 0
	if(g_reboot == 0)
	{
		D_PRINTF("First start!\n");
		return 0;
	}
	
	if(reset_adm())
	{
		printf("Stop IGMP Snooping failed!\n");
		return -1;
	}

	D_PRINTF("IGMP Snooping stopped!\n");
#endif
	return 0;
}
